' FlipMonitor.bas for FilmStar DESIGN
' Copyright 2018 FTG Software Associates
' Monitoring curves for AR deposited on both sides of a witness glass

Option Explicit
Option Base 1
DefInt i-N
DefSng A-H, O-Z
Const qDel# = .25     ' thickness interval

' *****************************************************
' NOTE: Monitor witness (massive layer) denoted by "G"
' NOTE: Requires DESIGN 2.61.4282 or newer
' NOTE: 2.61.4283 adds peak picker

Const wave = 550     ' monitor wavelength
' *****************************************************

Const sFile$ = "C:\Winfilm\Spectra\BBAR-Flip.xlsx"  ' XLS also works
Const dFile$ = "C:\Winfilm\Designs\BBAR-Flip.faw"   ' Design
Const tFile$ = "C:\Winfilm\Designs\BBAR-Flip-t.faw" ' Pseudo design for axes
Dim iMssv, kRow&, nLay, qm(), qm0(), qt$(), qd$(), qmLay#, qmTot#
Dim rp#, rs#, tp#, ts#

Sub Main
    Dim i, d0$, d1$, d2$, s$, qTot, nRows
    PlotSetup
    If FileExists(sFile$) Then Kill sFile$     ' Delete old monitor file
    DoEvents
    PlotClose ' in case left open
    FileOpen dFile$
    DoEvents
    d0$ = DesignReverseStr(Design)
    d1$ = StrReverse(Mid$(d0$, 1, InStr(d0$, "G")))
    d1$ = Mid$(d1$, InStr(d1$, " ") + 1)
    d2$ = DesignReverseStr(StrReverse(d1$)) & Mid$(d0$, InStr(d0$, "G") + 2)
    DoEvents
    GetLayers nLay, qm(), qt(), qd()    ' nLay original # layers including massive
    For i = 1 To nLay
        If qt$(i) = "G" Then
            iMssv = i
        Else
            qTot = qTot + qm(i)     ' total thickness
        End If
    Next i
    XLnew CLng(qTot/qDel) + 100, 3, "Monitor"  ' ensures sufficient rows
    XLwrite 1, 1, "t (nm)"
    XLwrite 1, 2, "%R"
    XLwrite 1, 3, "%T"
    SetLayersIgnore "ALL"   ' ignore all layers
    SetLayersIgnore ,iMssv  ' restore massive layer
    StatusLabel "Calculating Side 1"
    kRow = 1
    DoCalc
    SetLayersIgnore ' restore all layers
    Wait .5
    DesignReverse
    GetLayers nLay, qm(), qt(), qd()
    For i = iMssv + 1 To nLay
        s$ = s$ & CStr(i) & ", "
    Next i
    SetLayersIgnore Left$(s$, Len(s$) - 2)  ' ignore side 2 layers
    StatusLabel "Calculating Side 2"
    DoCalc
    XLsave sFile$
    XLclose: StatusLabel ""
    FileOpen tFile$
    DoEvents
    Design = d2$    ' pseudo design for monitor plot
    AxesCopy True
    s$ = Clipboard$()   ' adjust # of layers
    AxesPaste "1" & vbTab & CStr(nLay-1) & vbTab & "1" & vbTab & "1" & Mid$(s$, InStr(s$, vbCr))
    FileOpen sFile$
    DoEvents
    Replot: PlotActivate: PlotNext
End Sub

Sub DoCalc()
    Dim iLay, t#, qmTot0#
    For iLay = iMssv + 1 To nLay     ' deposit on 'right side' of substrate
        qmTot0 = qmTot
        SetLayersIgnore , iLay     ' remove ignore for layer i
        qmLay = qm(iLay)           ' thickness for layer i
        For t = 0 To qmLay Step qDel
            kRow = kRow + 1
            qm(iLay) = t
            SetLayers qm(), qt$()
            GetRTData wave, 0, rs, rp, ts, tp, True
            qmTot = qmTot0 + t
            XLwrite kRow, 1, qmTot, 3
            XLwrite kRow, 2, rs, 6
            XLwrite kRow, 3, ts, 6
        Next t
        qm(iLay) = qmLay   ' restore exact layer thickness
        SetLayers qm, qt$
        qmTot = qmTot0 + qmLay
    Next iLay
End Sub

Function FileExists(ByVal f$) As Boolean
    On Error GoTo NoFileFound
    FileExists = (FileLen(f$) > 0)
    Exit Function
NoFileFound:
    FileExists = False
    Exit Function
End Function

Sub PlotSetup
    ' FSPlot graph setup code for FilmStar BASIC:
    PlotConfig 8986817, 202128, 0
    PlotSize 11445, 8340, 65
    PlotTitleFont = "Arial"
    PlotTitleSize = 110
    PlotLegendSize = 120
    PlotLegendSimple = False
    PlotAxesFont = "Arial"
    PlotAxesLabelSize = 100
    PlotAxesNumberSize = 100
    PlotFontFixed = False
    PlotFontFixedSize = 1
    PlotAxesConfig = 0
    PlotEdgeSpace 50, -25, 50, 25
    PlotAxesSpace .15, .3, .3, 1.5
    PlotColors "255, 16711680, 57088, 16711935, 8421376, 32896, 0, 8421504, 128, 8388608, 32768, 8388736"
    PlotBitmapMode = True

    PlotBitmap(1) = ""
    PlotBitmap(2) = ""
    PlotBitmapType = 2
    PlotBitmapColors 13434828, 12648447, 12648447, 11513775, 0, 32896
    PlotTableColors 16777215, 0, 0, 0
End Sub
